/*
 * This file is part of the Poliqarp suite.
 * 
 * Copyright (C) 2009 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact
 * Michal.Ciesiolka@ipipan.waw.pl or ipi@ipipan.waw.pl for more
 * information).  Licensees holding a valid commercial license from IPI
 * PAN may use this file in accordance with that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */

#ifndef BPNG_XPATH_H
#define BPNG_XPATH_H 1

#include <bp/ng/xml-parser.h>
#include <libxml/xpath.h>

extern const char *xml_ns_poliqarp;

typedef xmlXPathContext xpath_context;
typedef xmlXPathObject xpath_object;

xpath_context *xpath_context_new(xml_document *, const char *id);

void xpath_context_free(xpath_context *);

void xpath_register_ns(xpath_context *, const char *prefix,
   const char *uri);

xpath_object *xpath_new(xpath_context *, const char *path);

void xpath_free(xpath_object *);

void xpath_register_functions(xpath_context *);

char *xpath_object_as_string(xpath_object *);

void xpath_cleanup();

#endif
